import type { Settings as LayoutSettings } from '@ant-design/pro-layout'
import { notification } from 'antd'
import { PageLoading } from '@ant-design/pro-layout'
import type { RequestConfig, RunTimeLayoutConfig } from 'umi'
import { Link, history } from 'umi'
import type { RequestOptionsInit, ResponseError } from 'umi-request'
import RightContent from '@/components/RightContent'
// import { currentUser as queryCurrentUser } from './services/ant-design-pro/api'
import consts from './utils/consts'
import { queryCurrentUser } from './services/api/login'

/** 获取用户信息比较慢的时候会展示一个 loading */
export const initialStateConfig = {
  loading: <PageLoading />
}

/**
 * @see  https://umijs.org/zh-CN/plugins/plugin-initial-state
 * */
export async function getInitialState(): Promise<{
  settings?: Partial<LayoutSettings>
  currentUser?: API.CurrentUser
  fetchUserInfo?: () => Promise<API.CurrentUser | undefined>
}> {
  const fetchUserInfo = async () => {
    try {
      const msg = await queryCurrentUser()
      return msg.data
    } catch (error) {
      history.push(consts.loginPath)
    }
    return undefined
  }
  // 如果是登录页面，不执行
  if (history.location.pathname !== consts.loginPath) {
    const currentUser = await fetchUserInfo()
    return {
      fetchUserInfo,
      ...currentUser,
      settings: {}
    }
  }
  return {
    fetchUserInfo,
    settings: {}
  }
}

const codeMaps = {
  // 200: '成功返回请求的数据。',
  201: '新建或修改数据成功。',
  202: '一个请求已经进入后台排队（异步任务）。',
  204: '删除数据成功。',
  400: '发出的请求有错误，服务器没有进行新建或修改数据的操作。',
  401: '用户没有权限（令牌、用户名、密码错误）。',
  403: '用户得到授权，但是访问是被禁止的。',
  404: '发出的请求针对的是不存在的记录，服务器没有进行操作。',
  406: '请求的格式不可得。',
  410: '请求的资源被永久删除，且不会再得到的。',
  422: '当创建一个对象时，发生一个验证错误。',
  500: '服务器发生错误，请检查服务器。',
  502: '网关错误。',
  503: '服务不可用，服务器暂时过载或维护。',
  504: '网关超时。'
}

const authHeaderInterceptor = (url: string, options: RequestOptionsInit) => {
  const token = window.localStorage.getItem('TOKEN_ID')
  // 如果是登录页面，不执行
  if (!token && history.location.pathname !== consts.loginPath) {
    return history.replace(consts.loginPath)
  }
  // const authHeader = { Authorization: `Bearer ${JSON.parse(token)}` }
  const authHeader = { Authorization: `Bearer ${token}` }
  return {
    url: `${url}`,
    options: { ...options, interceptors: true, headers: authHeader }
  }
}

// 处理非200的 http 状态码
const responseInterceptor = response => {
  codeMaps[response.status] &&
    notification.error({ message: response.status, description: codeMaps[response.status] })
  return response
}
export const request: RequestConfig = {
  errorHandler: (err: ResponseError) => {
    if (err.name === 'BizError') {
      const response = err.data
      if (response.code === consts.RET_CODE.SUCCESS) {
        return response
      }
      if (
        consts.TOKEN_INVALID_CODE.includes(response.code) &&
        window.location.pathname !== consts.loginPath
      ) {
        notification.error({
          message: '用户信息过期',
          description: '请重新登录'
        })
        history.replace({
          pathname: consts.loginPath,
          search: stringify({
            redirect: window.location.pathname
          })
        })
      }
      notification.error({
        message: '请求失败',
        description: response.msg
      })
      // 阻断代码向下执行
      return Promise.reject(response)
    }
    // 那么对于其他的错误, 需要实践中一个个找出来
    if (err.name === 'RequestError') {
      switch (err.type) {
        case 'Timeout': {
          message.error('系统超时,请联系开发人员')
        }
      }
    }

    return Promise.reject(err)
  },
  prefix: '/api',
  errorConfig: {
    adaptor: resData => {
      return {
        success: resData.code === consts.RET_CODE.SUCCESS,
        data: resData.data,
        errorCode: resData.code,
        errorMessage: resData.msg
      }
    }
  },
  responseInterceptors: [responseInterceptor],
  requestInterceptors: [authHeaderInterceptor]
}

// ProLayout 支持的api https://procomponents.ant.design/components/layout
export const layout: RunTimeLayoutConfig = ({ initialState }) => {
  return {
    rightContentRender: () => <RightContent />,
    disableContentMargin: false,
    // waterMarkProps: {
    //   content: initialState?.currentUser?.name
    // },
    onPageChange: () => {
      if (!initialState?.currentUser?.ID && location.pathname !== consts.loginPath) {
        history.replace({
          pathname: consts.loginPath,
          search: stringify({
            redirect: history.location.pathname
          })
        })
      }
    },
    menuHeaderRender: undefined,
    itemRender: (route, _, routes) => {
      if (routes.indexOf(route) === routes.length - 1) {
        return <span>{route.breadcrumbName}</span>
      }
      return <Link to={route.path}>{route.breadcrumbName}</Link>
    },
    // 自定义 403 页面
    // unAccessible: <div>unAccessible</div>,
    ...initialState?.settings
  }
}
